==> pfqueue-0.4.1.tar.gz: pfqueue-0.4.1/backends/pfq_exim.c <==

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>

#include "pfq_backend.h"
#include "pfq_service.h"
#include "../pfqmessage.h"

char spool_dir[BUF_SIZE];
char exim_cmd[BUF_SIZE];
char exim_conf[BUF_SIZE];

char *q_names[]={
	"input"
};

struct stat foostat;

int NUMMSG_THREAD;

#define HEADER_FROM	"From: "
#define HEADER_TO	"To: "
#define HEADER_SUBJECT  "Subject: "

const char* pfb_id() {
	return "exim";
}

const char* pfb_version() {
	return "1.1";
}

int queue_hasid ( const char* id, int dd ) {
	if ( strncmp ( ext_queue[dd].id, id, strlen(id) ) == 0 )
		return ext_queue[dd].cached;
	else
		return 0;
}

int dir_dig( const char* basedir ) {
	DIR *bdir;
	struct dirent *dir;
	char full_path[BUF_SIZE];
	struct msg_t *msg;
	int l;
	
	if ( NUMMSG_THREAD >= msg_max )
		return -1;

	if ( dig_limit && ( time(NULL)-dig_start > dig_limit ) )
		return -1;
	
	bdir = opendir ( basedir );
	while ( bdir && NUMMSG_THREAD < msg_max && ( dir = readdir ( bdir ) ) ) {

		if ( dig_limit && ( time(NULL)-dig_start > dig_limit ) )
			return -1;
	
		if ( dir->d_name[0] != '.' && ( dir->d_type == DT_DIR || dir->d_type == DT_LNK )) {
			snprintf ( full_path, sizeof(full_path), "%s/%s", basedir, dir->d_name );
			dir_dig ( full_path );
		} else if ( NUMMSG_THREAD < msg_max && dir->d_type == DT_REG ) {
			l = strlen(dir->d_name);
			if ( dir->d_name[l-1]=='H' && dir->d_name[l-2]=='-') {
				msg = &(my_queue[NUMMSG_THREAD]);
				memcpy ( msg->id, dir->d_name, l-2 );
				snprintf ( msg->path, sizeof(msg->path), "%s/%s",
					basedir, dir->d_name );
				msg->cached = queue_hasid(msg->id, NUMMSG_THREAD);
				NUMMSG_THREAD++;
			}
		} else if ( dir->d_name[0] != '.' && dir->d_type == DT_UNKNOWN ) {
			snprintf ( full_path, sizeof(full_path), "%s/%s", basedir, dir->d_name );
			stat ( full_path, &foostat );
			if ( S_ISDIR(foostat.st_mode) || S_ISLNK(foostat.st_mode)) {
				dir_dig ( full_path );
			} else if ( NUMMSG_THREAD < msg_max && S_ISREG(foostat.st_mode) ) {
				l = strlen(dir->d_name);
				if ( dir->d_name[l-1]=='H' && dir->d_name[l-2]=='-') {
					msg = &(my_queue[NUMMSG_THREAD]);
					memcpy ( msg->id, dir->d_name, l-2 );
					snprintf ( msg->path, sizeof(msg->path), "%s/%s",
						basedir, dir->d_name );
					msg->cached = queue_hasid(msg->id, NUMMSG_THREAD);
					NUMMSG_THREAD++;
				}
			}
		}
		
	}
	if ( bdir )
		closedir ( bdir );

	return 0;
}


int pfb_init( int argc, char** argv ) {
	FILE *p;
	char buf[BUF_SIZE];
	int opt;

	strcpy ( exim_cmd, "exim");
	strcpy ( exim_conf, "" );
	
	pfb_caps = BECAPS_MSG_HOLD + BECAPS_MSG_DEL + BECAPS_MSG_REQUEUE + BECAPS_MIXED_QUEUE +
		BECAPS_MSG_LOG;

	opt = 0;
	while ( opt != -1 ) {
		opt = getopt ( argc, argv, "p:c:" );
		switch ( opt ) {
		case 'p':
			if ( strlen(optarg) ) 
				snprintf ( exim_cmd, BUF_SIZE-1, "%s/exim", optarg );
			break;
		case 'c':
			if ( strlen(optarg) )
				snprintf ( exim_conf, BUF_SIZE-1, " -C %s ", optarg );
			break;
		}
	}

	// Try exim 3...
	snprintf ( buf, BUF_SIZE, "%s %s -bP spool_directory 2> /dev/null |cut -d'=' -f2|cut -c2-", 
			exim_cmd, exim_conf );
	p = popen ( buf, "r" );
	if ( p ) {
		freadl ( p, spool_dir, sizeof(spool_dir) );
		pclose ( p );
	}

	// or exim 4
	if ( !strlen(spool_dir) ) {
		strcpy ( exim_cmd, "exim4");
		snprintf ( buf, BUF_SIZE, "%s %s -bP spool_directory 2> /dev/null |cut -d'=' -f2|cut -c2-", 
			exim_cmd, exim_conf );
		p = popen ( buf, "r" );
		if ( p ) {
			freadl ( p, spool_dir, sizeof(spool_dir) );
			pclose ( p );
		}
	}
	if ( !strlen(spool_dir) ) {
		fprintf ( stderr, "exim pfqueue backend: cannot guess spool_directory\n" );
		return -1;
	}

	return 0;
}

int pfb_close() {
	return 0;
}

int pfb_setup ( int maxmsg, int maxtime, struct msg_t* qptr1, struct msg_t* qptr2 ) {
	msg_max = maxmsg;
	dig_limit = maxtime;
	ext_queue = qptr1;
	my_queue = qptr2;
	return 0;
}

int pfb_fill_queue() {
	char pbuf[BUF_SIZE];
	NUMMSG_THREAD = 0;
	snprintf ( pbuf, BUF_SIZE, "%s/input", spool_dir );
	dir_dig( spool_dir );
	return NUMMSG_THREAD;
}

int pfb_retr_headers( const int msgp ) {
	FILE *p;
	char buf[BUF_SIZE];
	int f1, f2, f3, l1, l2;
	char *s1, *s2;
	struct msg_t *msg;
	
	msg = &ext_queue[msgp];
	
	snprintf ( buf, BUF_SIZE, "%s %s -Mvh %s 2> /dev/null", 
		exim_cmd, exim_conf, msg->id );

	p = popen ( buf, "r" );
	if ( !p ) {
		strcpy ( msg->from, "cant popen" );
		strcpy ( msg->to,   "cant popen" );
		msg->cached = 1;
		return 2;
	}
	
	f1 = f2 = f3 = 0;
	
	l1 = strlen(HEADER_FROM);
	l2 = strlen(HEADER_TO);
	s1 = HEADER_FROM;
	s2 = HEADER_TO;

	strcpy ( msg->stat, "Active" );
	
	while ( !msg->cached && (f1==0||f2==0||f3==0) && freadl ( p, buf, BUF_SIZE ) ) {
		if ( !strncmp ( buf, "-frozen", 7 ) )
			strcpy ( msg->stat, "Frozen" );
		if ( !f1 && ( !strncmp ( buf+5, s1, l1 ) ) ) {
			memcpy ( msg->from, buf+l1+5, sizeof(msg->from) );
			if ( !strlen(msg->from) )
				strcpy ( msg->from, "Null sender" );
			f1++;
		}
		if ( !f2 && ( !strncmp ( buf+5, s2, l2 ) ) ) {
			memcpy ( msg->to, buf+l2+5, sizeof(msg->to) );
			f2++;
		}
		if ( !f3 && !strncmp ( buf+5, HEADER_SUBJECT, strlen(HEADER_SUBJECT) ) ) {
			memcpy ( msg->subj, buf+strlen(HEADER_SUBJECT)+5, sizeof(msg->subj) );
			f3++;
		}
	}
	while ( msg->cached && (f1==0) && freadl ( p, buf, BUF_SIZE ) ) {
		if ( !strncmp ( buf, "-frozen", 7 ) ) {
			strcpy ( msg->stat, "Frozen" );
			f1=1;
		}
	}
	pclose ( p );

	if ( strlen(msg->to) )
		msg->cached = 1;

	return 0;
}

int pfb_retr_body( int msg, char *buffer, size_t buflen ) {
	char b[BUF_SIZE];
	int j;
	FILE *p;

	snprintf ( b, BUF_SIZE, "%s %s -Mvb %s 2> /dev/null", 
		exim_cmd, exim_conf, ext_queue[msg].id );
	
	p = popen ( b, "r" );
	if ( !p )
		return -1;

	// Skip first line
	freadl ( p, b, sizeof(b) );
	j = fread( buffer, sizeof(char), buflen, p );
	pclose ( p );
	return j;
}

int pfb_action(int act, int msg) {
char b[BUF_SIZE];
char buf[BUF_SIZE];

	switch ( act ) {
	case MSG_DELETE:
		strcpy ( b, "-Mrm" );
		break;
	case MSG_HOLD:
		strcpy ( b, "-Mf" );
		break;
	case MSG_RELEASE:
		strcpy ( b, "-Mt" );
		break;
	case MSG_REQUEUE:
		strcpy ( b, "-M" );
		break;
	default:
		return 1;
	}

	snprintf ( buf, BUF_SIZE, "%s %s %s %s > /dev/null",
			exim_cmd, exim_conf, b, ext_queue[msg].id );
	system ( buf );
	return 0;
}

int pfb_message_delete( int msg ) {
	return pfb_action ( MSG_DELETE, msg );
}

int pfb_message_hold( int msg ) {
	return pfb_action ( MSG_HOLD, msg );
}

int pfb_message_requeue( int msg ) {
	return pfb_action ( MSG_REQUEUE, msg );
}

int pfb_message_release( int msg ) {
	return pfb_action ( MSG_RELEASE, msg );
}

int pfb_set_queue ( int q ) {
	return 0;
}

void pfb_use_envelope ( int u ) {
}

int pfb_get_caps() {
	return pfb_caps;
}

char* pfb_queue_get_name ( int q ) {
	return q_names[q];
}

int pfb_queue_count() {
	return 1;
}

==> pfqueue-0.4.2.tar.gz: pfqueue-0.4.2/backends/pfq_exim.c <==

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>

#include "pfq_backend.h"
#include "pfq_service.h"
#include "../pfqmessage.h"
#include "../config.h"

char spool_dir[BUF_SIZE];
char exim_cmd[BUF_SIZE];
char exim_conf[BUF_SIZE];

char *q_names[]={
	"input"
};

int NUMMSG_THREAD;

#define HEADER_FROM	"From: "
#define HEADER_TO	"To: "
#define HEADER_SUBJECT  "Subject: "

const char* pfb_id() {
	return "exim";
}

const char* pfb_version() {
	return "1.2";
}

int dir_dig( const char* basedir ) {
	DIR *bdir;
	struct dirent *dir;
	char full_path[BUF_SIZE];
	struct be_msg_t *msg;
	int l;
	
	if ( NUMMSG_THREAD >= msg_max )
		return -1;

	if ( dig_limit && ( time(NULL)-dig_start > dig_limit ) )
		return -1;
	
	bdir = opendir ( basedir );
	while ( bdir && NUMMSG_THREAD < msg_max && ( dir = readdir ( bdir ) ) ) {

		if ( dig_limit && ( time(NULL)-dig_start > dig_limit ) )
			return -1;
		snprintf ( full_path, sizeof(full_path), "%s/%s", basedir, dir->d_name );
	
		if ( fs_should_dig ( dir, full_path ) ) {
			dir_dig ( full_path );
		} else if ( NUMMSG_THREAD < msg_max && fs_should_add ( dir, full_path ) ) {
			l = strlen(dir->d_name);
			if ( dir->d_name[l-1]=='H' && dir->d_name[l-2]=='-') {
				msg = &(my_queue[NUMMSG_THREAD]);
				memcpy ( msg->id, dir->d_name, l-2 );
				snprintf ( msg->path, sizeof(msg->path), "%s/%s",
					basedir, dir->d_name );
				msg->changed = strcmp (dir->d_name, ext_queue[NUMMSG_THREAD].id );
				NUMMSG_THREAD++;
			}
		}
	}
	if ( bdir )
		closedir ( bdir );

	return 0;
}


int pfb_init( int argc, char** argv ) {
	FILE *p;
	char buf[BUF_SIZE];
	int opt;

	strcpy ( exim_cmd, "exim");
	strcpy ( exim_conf, "" );
	strcpy ( spool_dir, "" );
	
	pfb_caps = BECAPS_MSG_HOLD + BECAPS_MSG_DEL + BECAPS_MSG_REQUEUE + BECAPS_MIXED_QUEUE +
		BECAPS_MSG_LOG;

	opt = 0;
	while ( opt != -1 ) {
		opt = getopt ( argc, argv, "p:c:" );
		switch ( opt ) {
		case 'p':
			if ( strlen(optarg) ) 
				snprintf ( exim_cmd, BUF_SIZE-1, "%s/exim", optarg );
			break;
		case 'c':
			if ( strlen(optarg) )
				snprintf ( exim_conf, BUF_SIZE-1, " -C %s ", optarg );
			break;
		}
	}

	// Try exim 3...
	snprintf ( buf, BUF_SIZE, "%s %s -bP spool_directory 2> /dev/null |cut -d'=' -f2|cut -c2-", 
			exim_cmd, exim_conf );
	p = popen ( buf, "r" );
	if ( p ) {
		freadl ( p, spool_dir, sizeof(spool_dir) );
		pclose ( p );
	}

	// or exim 4
	if ( !strlen(spool_dir) ) {
		strcpy ( exim_cmd, "exim4");
		snprintf ( buf, BUF_SIZE, "%s %s -bP spool_directory 2> /dev/null |cut -d'=' -f2|cut -c2-", 
			exim_cmd, exim_conf );
		p = popen ( buf, "r" );
		if ( p ) {
			freadl ( p, spool_dir, sizeof(spool_dir) );
			pclose ( p );
		}
	}
	if ( !strlen(spool_dir) ) {
		fprintf ( stderr, "exim pfqueue backend: cannot guess spool_directory\n" );
		return -1;
	}

	return 0;
}

int pfb_close() {
	return 0;
}

int pfb_setup ( int maxmsg, int maxtime, struct msg_t* qptr1, struct be_msg_t* qptr2 ) {
	msg_max = maxmsg;
	dig_limit = maxtime;
	ext_queue = qptr1;
	my_queue = qptr2;
	return 0;
}

int pfb_fill_queue() {
	char pbuf[BUF_SIZE];
	NUMMSG_THREAD = 0;
	snprintf ( pbuf, BUF_SIZE, "%s/input", spool_dir );
	dir_dig( spool_dir );
	return NUMMSG_THREAD;
}

int pfb_retr_headers( const int msgp ) {
	FILE *p;
	char buf[BUF_SIZE];
	int f1, f2, f3, l1, l2;
	char *s1, *s2;
	struct msg_t *msg;
	
	msg = &ext_queue[msgp];
	
	snprintf ( buf, BUF_SIZE, "%s %s -Mvh %s 2> /dev/null", 
		exim_cmd, exim_conf, msg->id );

	p = popen ( buf, "r" );
	if ( !p ) {
		strcpy ( msg->from, "cant popen" );
		strcpy ( msg->to,   "cant popen" );
		msg->hcached = 1;
		return 2;
	}
	
	f1 = f2 = f3 = 0;
	
	l1 = strlen(HEADER_FROM);
	l2 = strlen(HEADER_TO);
	s1 = HEADER_FROM;
	s2 = HEADER_TO;

	while ( !msg->hcached && (f1==0||f2==0||f3==0) && freadl ( p, buf, BUF_SIZE ) ) {
		if ( !f1 && ( !strncmp ( buf+5, s1, l1 ) ) ) {
			memcpy ( msg->from, buf+l1+5, sizeof(msg->from) );
			if ( !strlen(msg->from) )
				strcpy ( msg->from, "Null sender" );
			f1++;
		}
		if ( !f2 && ( !strncmp ( buf+5, s2, l2 ) ) ) {
			memcpy ( msg->to, buf+l2+5, sizeof(msg->to) );
			f2++;
		}
		if ( !f3 && !strncmp ( buf+5, HEADER_SUBJECT, strlen(HEADER_SUBJECT) ) ) {
			memcpy ( msg->subj, buf+strlen(HEADER_SUBJECT)+5, sizeof(msg->subj) );
			f3++;
		}
	}
	pclose ( p );

	if ( strlen(msg->to) && strlen(msg->from) )
		msg->hcached = 1;

	return 0;
}

int pfb_retr_status( const int msgp ) {
	FILE *p;
	char buf[BUF_SIZE];
	int f1;
	struct msg_t *msg;
	
	msg = &ext_queue[msgp];
	
	snprintf ( buf, BUF_SIZE, "%s %s -Mvh %s 2> /dev/null", 
		exim_cmd, exim_conf, msg->id );

	p = popen ( buf, "r" );
	if ( !p ) {
		strcpy ( msg->stat, "cant popen" );
		return 2;
	}
	
	f1 = 0;
	
	strcpy ( msg->stat, "Active" );
	while ( f1==0 && freadl ( p, buf, BUF_SIZE ) ) {
		if ( !strncmp ( buf, "-frozen", 7 ) )
			strcpy ( msg->stat, "Frozen" );
	}
	pclose ( p );
	msg->scached = 0;

	return 0;
}
int pfb_retr_body( int msg, char *buffer, size_t buflen ) {
	char b[BUF_SIZE];
	int j;
	FILE *p;

	snprintf ( b, BUF_SIZE, "%s %s -Mvb %s 2> /dev/null", 
		exim_cmd, exim_conf, ext_queue[msg].id );
	
	p = popen ( b, "r" );
	if ( !p )
		return -1;

	// Skip first line
	freadl ( p, b, sizeof(b) );
	j = fread( buffer, sizeof(char), buflen, p );
	pclose ( p );
	return j;
}

int pfb_action(int act, int msg) {
char b[BUF_SIZE];
char buf[BUF_SIZE];

	switch ( act ) {
	case MSG_DELETE:
		strcpy ( b, "-Mrm" );
		break;
	case MSG_HOLD:
		strcpy ( b, "-Mf" );
		break;
	case MSG_RELEASE:
		strcpy ( b, "-Mt" );
		break;
	case MSG_REQUEUE:
		strcpy ( b, "-M" );
		break;
	default:
		return 1;
	}

	snprintf ( buf, BUF_SIZE, "%s %s %s %s > /dev/null",
			exim_cmd, exim_conf, b, ext_queue[msg].id );
	system ( buf );
	return 0;
}

int pfb_message_delete( int msg ) {
	return pfb_action ( MSG_DELETE, msg );
}

int pfb_message_hold( int msg ) {
	return pfb_action ( MSG_HOLD, msg );
}

int pfb_message_requeue( int msg ) {
	return pfb_action ( MSG_REQUEUE, msg );
}

int pfb_message_release( int msg ) {
	return pfb_action ( MSG_RELEASE, msg );
}

int pfb_set_queue ( int q ) {
	return 0;
}

void pfb_use_envelope ( int u ) {
}

int pfb_get_caps() {
	return pfb_caps;
}

char* pfb_queue_get_name ( int q ) {
	return q_names[q];
}

int pfb_queue_count() {
	return 1;
}
